<option value="{{ widget.value }}" {% if widget.parent %}data-pup="{{widget.parent}}"{% endif %} class="l{% if widget.level %}{{widget.level}} {% if not widget.leaf %} non-leaf{% endif %}{% else %}1{% endif %}" {% include "django/forms/widgets/attrs.html" %}>{{ widget.label }}</option>
